import {
  AfterViewInit,
  Component,
  ElementRef,
  forwardRef,
  Input,
  Renderer2,
  ViewChild,
} from "@angular/core";
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from "@angular/forms";
import { NbpCalendarPattern, NbpCalendarPosition } from "@isp/xdce-widget";

@Component({
  selector: 'nbp-fid-calendar-generic',
  templateUrl: './nbp-fid-calendar-genric.component.html',
  styleUrls: ["./nbp-fid-calendar-genric.component.scss"],
  providers: [
    {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => NbpFidCalendarGenericComponent),
        multi: true
      }
  ]
})
export class NbpFidCalendarGenericComponent implements AfterViewInit, ControlValueAccessor  {
  @Input("nbpLabelPattern") pattern: NbpCalendarPattern =
    NbpCalendarPattern.GGMMAAAA;
  @Input("nbpPlacement") placement: string = "bottom-right";
  @Input("id") id: string;
  @Input("name") name: string;
  daysOfWeek = ["lun", "mar", "mer", "gio", "ven", "sab", "dom"];
  months = [
    "Gennaio",
    "Febbraio",
    "Marzo",
    "Aprile",
    "Maggio",
    "Giugno",
    "Luglio",
    "Agosto",
    "Settembre",
    "Ottobre",
    "Novembre",
    "Dicembre",
  ];
  view: "days" | "months" | "years" = "days";

  visibility: "shown" | "hidden" = "hidden";

  disabledWeekdays = [0, 6]; // es: weekend
  // CONFIGURAZIONE: range di date da disabilitare (array di oggetti {start, end})
  disabledDateRanges = [
    { start: new Date(2025, 4, 10), end: new Date(2025, 4, 15) }, // 10-15 maggio 2025
    // aggiungi altri range se necessario
  ];

  ngAfterViewInit(): void {
    this.renderer.setStyle(
      this.calendarPopupRef.nativeElement,
      "display",
      "none"
    );
  }
  constructor(private renderer: Renderer2) {}

  

  currentDate = new Date();
  selectedDate: Date | null = null;
  selectedDateStr : string | null = null;
  monthYearText: string = "";
  firstDayOfMonth: Date | null = null;
  startDay: number = 0;
  daysInCurrentMonth: number = 0;
  daysInCurrentMonthToBeRendered: Date[] = [];
  daysInPrevMonth: number = 0;
  daysInPrevMonthToBeRendered: Date[] = [];
  daysInNextMonthToBeRendered: Date[] = [];
  monthsToBeRendered: string[] = [];
  yearsToBeRendered: string[] = [];

  @ViewChild("dateInput") dateInputRef: ElementRef;
  @ViewChild("calendarPopup") calendarPopupRef: ElementRef;

  public showCalendar() {
    this.positionCalendarPopup();
    this.renderCalendar();
  }

  public hideCalendar() {
    this.renderer.setStyle(
      this.calendarPopupRef.nativeElement,
      "display",
      "none"
    );
  }

  private renderCalendar() {
    if (this.view == "days") this.renderDays();
    else if (this.view == "months") this.renderMonths();
    else this.renderYears();
  }

  private positionCalendarPopup(): void {
    const input = this.dateInputRef.nativeElement;
    const popup = this.calendarPopupRef.nativeElement;

    // Append to body if not already
    if (popup.parentNode !== document.body) {
      this.renderer.appendChild(document.body, popup);
    }

    const rect = input.getBoundingClientRect();
    const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
    const scrollLeft =
      window.pageXOffset || document.documentElement.scrollLeft;

    this.renderer.setStyle(popup, "display", "block");
    this.renderer.setStyle(popup, "position", "absolute");
    this.renderer.setStyle(popup, "z-index", "9999");
    this.renderer.setStyle(popup, "top", `${rect.bottom + scrollTop}px`);
    this.renderer.setStyle(popup, "left", `${rect.left + scrollLeft}px`);
    // this.renderer.setStyle(popup, 'min-width', `${rect.width}px`);
  }
  public pad(n: number): string {
    return n < 10 ? "0" + n : n + "";
  }
  public isDateDisabled(date: Date): boolean {
    // Disabilita per giorno della settimana
    if (this.disabledWeekdays.indexOf(date.getDay()) != -1) return true;
    // Disabilita per range
    for (const range of this.disabledDateRanges) {
      if (date >= range.start && date <= range.end) return true;
    }
    return false;
  }
  public sameDate(d1: Date, d2: Date) {
    return (
      d1.getFullYear() === d2.getFullYear() &&
      d1.getMonth() === d2.getMonth() &&
      d1.getDate() === d2.getDate()
    );
  }

  public sameMonth(m1: number, m2: number) {
    return m1 == m2;
  }

  public sameYear(y1: number, y2: number) {
    return y1 == y2;
  }

  private formatDate(date: Date, format: string) {
    if (!date) return "";
    return format
      .replace("gg", this.pad(date.getDate()))
      .replace("mm", this.pad(date.getMonth() + 1))
      .replace("aaaa", date.getFullYear().toString())
      .replace("aa", String(date.getFullYear()).slice(-2));
  }

  public selectDate(date: Date) {
    if (this.isDateDisabled(date)) {
      return;
    }
    this.selectedDate = date;
    this.setDate(this.formatDate(date, this.pattern));
    this.renderCalendar();
  }

  public selectMonth(idx) {
    this.currentDate.setMonth(idx);
    this.view = "days";
    this.renderCalendar();
  }

  public selectYear(year: string) {
    this.currentDate.setFullYear(Number(year));
    this.view = "months";
    this.renderCalendar();
  }

  public isToday(date: Date) {
    return this.sameDate(date, new Date());
  }

  public isCurrentMonth(idx) {
    return idx == this.currentDate.getMonth();
  }

  public isCurrentYear(year: string) {
    return year === this.currentDate.getFullYear().toString();
  }

  private renderDays() {
    this.daysInPrevMonthToBeRendered = [];
    this.daysInCurrentMonthToBeRendered = [];
    this.daysInNextMonthToBeRendered = [];

    this.monthYearText = `${
      this.months[this.currentDate.getMonth()]
    } ${this.currentDate.getFullYear()}`;

    const year = this.currentDate.getFullYear();
    const month = this.currentDate.getMonth();

    this.firstDayOfMonth = new Date(year, month, 1);
    this.startDay = (this.firstDayOfMonth.getDay() + 6) % 7;

    this.daysInCurrentMonth = new Date(year, month + 1, 0).getDate();
    this.daysInPrevMonth = new Date(year, month, 0).getDate();

    for (let i = this.startDay; i > 0; i--) {
      const day = this.daysInPrevMonth - i + 1;
      const prevMonthDate = new Date(year, month - 1, day);
      this.daysInPrevMonthToBeRendered.push(prevMonthDate);
    }

    for (let i = 1; i <= this.daysInCurrentMonth; i++) {
      const currMonthDate = new Date(year, month, i);
      this.daysInCurrentMonthToBeRendered.push(currMonthDate);
    }

    const totalCells = 42; // 7 giorni x 6 settimane
    const filledCells = this.startDay + this.daysInCurrentMonth;
    const nextMonthDays = totalCells - filledCells;
    for (let i = 1; i <= nextMonthDays; i++) {
      const nextMonthDate = new Date(year, month + 1, i);
      this.daysInNextMonthToBeRendered.push(nextMonthDate);
    }
  }

  private renderMonths() {
    this.monthsToBeRendered = [];
    const monthTexts = this.months.map((m) => m.substring(0, 3));
    this.monthYearText = this.currentDate.getFullYear().toString();
    monthTexts.forEach((m, idx) => {
      this.monthsToBeRendered.push(m);
    });
  }

  private renderYears() {
    this.yearsToBeRendered = [];
    // 20 anni, 5x4
    const base = Math.floor(this.currentDate.getFullYear() / 20) * 20;
    this.monthYearText = `${base} - ${base + 19}`;
    for (let y = base; y < base + 20; y++) {
      this.yearsToBeRendered.push(y.toString());
    }
  }

  public changeView() {
    if (this.view === "days") this.view = "months";
    else if (this.view === "months") this.view = "years";
    this.renderCalendar();
  }

  public prev() {
    if (this.view === "days")
      this.currentDate.setMonth(this.currentDate.getMonth() - 1);
    else if (this.view === "months")
      this.currentDate.setFullYear(this.currentDate.getFullYear() - 1);
    else this.currentDate.setFullYear(this.currentDate.getFullYear() - 10);
    this.renderCalendar();
  }

  public next() {
    if (this.view === "days")
      this.currentDate.setMonth(this.currentDate.getMonth() + 1);
    else if (this.view === "months")
      this.currentDate.setFullYear(this.currentDate.getFullYear() + 1);
    else this.currentDate.setFullYear(this.currentDate.getFullYear() + 10);
    this.renderCalendar();
  }

  public goToday() {
    this.currentDate = new Date();
    this.selectedDate = new Date();
    this.setDate(this.formatDate(
      this.selectedDate,
      this.pattern
    ));
    this.view = "days";
    this.renderCalendar();
  }

  public clearSelection() {
    this.selectedDate = null;
    this.setDate("")
    this.renderCalendar();
  }

  public confirmDate() {
    this.hideCalendar();
  }

  private onChange = (_: any) => {};
  private onTouched = () => {};

  writeValue(value: string | null): void {
    this.selectedDateStr = value;
  }

  registerOnChange(fn: any): void {
    this.onChange = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouched = fn;
  }

  setDate(value: string): void {
    this.selectedDateStr = value;
    this.onChange(value);
    this.onTouched();
  }
    
    setDisabledState?(isDisabled: boolean): void {
        throw new Error("Method not implemented.");
    }
}
